@@ -20,5 +20,5 @@ You should have received a copy of the GNU General Public License
 along with GCC; see the file COPYING3.  If not see
 <http://www.gnu.org/licenses/>.  */
 
-#define TARGET_64BIT_DEFAULT MASK_64BIT
+#define TARGET_64BIT_DEFAULT OPTION_MASK_ISA_64BIT
 #define TARGET_BI_ARCH 1
