@@ -5,5 +5,5 @@
 /* { dg-options "" } */
 
 int f(int (*)[]);
-void g() { int f(int (*)[2]); } /* { dg-error "error: previous declaration of 'f' was here" } */
-int f(int (*)[3]); /* { dg-error "error: conflicting types for 'f'" } */
+void g() { int f(int (*)[2]); } /* { dg-error "previous declaration of 'f' was here" } */
+int f(int (*)[3]); /* { dg-error "conflicting types for 'f'" } */
