@@ -606,25 +606,64 @@ replace_def_variable (var_map map, def_operand_p def_p, tree *expr)
 }
 
 
-/* Remove any PHI node which is a virtual PHI.  */
+/* Remove each argument from PHI.  If an arg was the last use of an SSA_NAME, 
+   check to see if this allows another PHI node to be removed.  */
 
 static void
-eliminate_virtual_phis (void)
+remove_tree_phi_args (tree phi)
+{
+  use_operand_p arg_p;
+  ssa_op_iter iter;
+
+  if (dump_file && (dump_flags & TDF_DETAILS))
+    {
+      fprintf (dump_file, "Removing Dead PHI definition: ");
+      print_generic_stmt (dump_file, phi, TDF_SLIM);
+    }
+
+  FOR_EACH_PHI_ARG (arg_p, phi, iter, SSA_OP_USE)
+    {
+      tree arg = USE_FROM_PTR (arg_p);
+      if (TREE_CODE (arg) == SSA_NAME)
+        {
+	  /* Remove the reference to the existing argument.  */
+	  SET_USE (arg_p, NULL_TREE);
+	  if (has_zero_uses (arg))
+	    {
+	      tree stmt = SSA_NAME_DEF_STMT (arg);
+
+	      /* Also remove the def if it is a PHI node.  */
+	      if (TREE_CODE (stmt) == PHI_NODE)
+		{
+		  remove_tree_phi_args (stmt);
+		  remove_phi_node (stmt, NULL_TREE, true);
+		}
+	    }
+	}
+    }
+}
+
+
+/* Remove any PHI node which is a virtual PHI, or a PHI with no uses.  */
+
+static void
+eliminate_useless_phis (void)
 {
   basic_block bb;
-  tree phi, next;
+  tree result, next, phi;
 
   FOR_EACH_BB (bb)
     {
-      for (phi = phi_nodes (bb); phi; phi = next)
+      for (phi = phi_nodes (bb); phi; phi = next) 
         {
 	  next = PHI_CHAIN (phi);
-	  if (!is_gimple_reg (SSA_NAME_VAR (PHI_RESULT (phi))))
+	  result = PHI_RESULT (phi);
+	  if (!is_gimple_reg (SSA_NAME_VAR (result)))
 	    {
 #ifdef ENABLE_CHECKING
 	      int i;
-	      /* There should be no arguments of this PHI which are in
-		 the partition list, or we get incorrect results.  */
+	      /* There should be no arguments which are not virtual, or the
+	         results will be incorrect.  */
 	      for (i = 0; i < PHI_NUM_ARGS (phi); i++)
 	        {
 		  tree arg = PHI_ARG_DEF (phi, i);
@@ -639,7 +678,16 @@ eliminate_virtual_phis (void)
 		    }
 		}
 #endif
-	      remove_phi_node (phi, NULL_TREE, true);
+	      remove_phi_node (phi, NULL_TREE , true);
+	    }
+          else
+	    {
+	      /* Also remove real PHIs with no uses.  */
+	      if (has_zero_uses (result))
+	        {
+		  remove_tree_phi_args (phi);
+		  remove_phi_node (phi, NULL_TREE, true);
+		}
 	    }
 	}
     }
@@ -1449,7 +1497,9 @@ rewrite_out_of_ssa (void)
      copies into the loop itself.  */
   insert_backedge_copies ();
 
-  eliminate_virtual_phis ();
+
+  /* Eliminate PHIs which are of no use, such as virtual or dead phis.  */
+  eliminate_useless_phis ();
 
   if (dump_file && (dump_flags & TDF_DETAILS))
     dump_tree_cfg (dump_file, dump_flags & ~TDF_DETAILS);
