@@ -28,8 +28,10 @@
 // invalidate any other reasons why the executable file might be covered by
 // the GNU General Public License.
 
-#define debug 0
-
-#include <debug/vector>
+#include <vector>
 
 // libstdc++26412
+namespace debug
+{
+  int i;
+}
