@@ -42,34 +42,32 @@ void test01()
   basic_filebuf<pod_uchar> fb;
   fb.pubimbue(loc);
 
-  fb.open(name, ios_base::out | ios_base::trunc);
+  fb.open(name, ios_base::in | ios_base::out | ios_base::trunc);
   fb.sputc(pod_uchar::from<char>('b'));
   fb.sputc(pod_uchar::from<char>(0xff));
+  fb.sputc(pod_uchar::from<char>('c'));
+  fb.sputc(pod_uchar::from<char>(0xff));
+
+  fb.pubseekoff(0, ios_base::beg);
+  fb.sbumpc();
+  fb.sbumpc();
+  traits_type::pos_type pos = fb.pubseekoff(0, ios_base::cur);
+  fb.sbumpc();
+  fb.sbumpc();
+
+  // Check that seekoff sets the current state for output
+  fb.pubseekpos(pos);
+
   fb.sputc(pod_uchar::from<char>('a'));
-  fb.sputc(pod_uchar::from<char>(0xfc));
   fb.sputc(pod_uchar::from<char>(0));
   fb.sputc(pod_uchar::from<char>(0));
+  fb.sputc(pod_uchar::from<char>(0));
+  fb.sputc(pod_uchar::from<char>(0));
+  
+  fb.pubseekpos(pos);
 
-  fb.close();
-  fb.open(name, ios_base::in);
-
-  fb.sbumpc();
-  fb.sbumpc();
-  traits_type::pos_type pos1 = fb.pubseekoff(0, ios_base::cur);
-  fb.sbumpc();
-  fb.sbumpc();
-
-  // Check that seekpos returns the correct state
-  traits_type::pos_type pos2 = fb.pubseekpos(pos1);
   traits_type::int_type c = fb.sbumpc();
   VERIFY( c != traits_type::eof() );
-  VERIFY( traits_type::eq(traits_type::to_char_type(c),
-			  pod_uchar::from<char>('a')) );
-  fb.sbumpc();
-
-  fb.pubseekpos(pos2);
-  c = fb.sbumpc();
-  VERIFY( c != traits_type::eof() );
   VERIFY( traits_type::eq(traits_type::to_char_type(c),
 			  pod_uchar::from<char>('a')) );
 
