@@ -16,28 +16,22 @@
 // Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 // USA.
 
-// The ARM simulator does not provide support for "fstat", which
-// causes "in_avail" to return an incorrect value.
-// { dg-do run { xfail arm*-*-elf arm*-*-eabi } }
-
 // 27.6.1.3 unformatted input functions
-// @require@ %-*.tst %-*.txt
-// @diff@ %-*.tst %-*.txt
 
 #include <istream>
-#include <fstream>
+#include <sstream>
 #include <testsuite_hooks.h>
 
 // libstdc++/6746   
-void test13()
+void test12()
 {
   using namespace std;
   bool test __attribute__((unused)) = true;
   streamsize sum = 0;
-  wifstream ifs("wistream_unformatted-1.tst");
+  wistringstream iss(L"shamma shamma");
       
   // test01
-  size_t i = ifs.rdbuf()->in_avail();
+  size_t i = iss.rdbuf()->in_avail();
   VERIFY( i != 0 );
     
   // test02
@@ -45,16 +39,16 @@ void test13()
   do
     {
       wchar_t buf[1024];
-      extracted = ifs.readsome(buf, sizeof(buf) / sizeof(wchar_t));
+      extracted = iss.readsome(buf, sizeof(buf) / sizeof(wchar_t));
       sum += extracted;
     }
-  while (ifs.good() && extracted);
-  VERIFY( sum != 0 );  
+  while (iss.good() && extracted);
+  VERIFY( sum != 0 );
 }
- 
+
 int 
 main()
 {
-  test13();
+  test12();
   return 0;
 }
